from __future__ import unicode_literals

from telesign.rest import RestClient
from telesignenterprise.constants import SOURCE_SDK
import telesignenterprise
import telesign
import requests, json, base64

BASE_URL_VERIFY_API = "https://verify.telesign.com"
PATH_VERIFICATION_CREATE = "/verification"
PATH_VERIFICATION_RETRIEVE = "/verification/{reference_id}"
PATH_VERIFICATION_UPDATE = "/verification/{reference_id}/state"

class OmniVerify(RestClient):
    """
    OmniVerify class to handle omnichannel verification API calls.
    """

    def __init__(self, customer_id, api_key, rest_endpoint=BASE_URL_VERIFY_API, **kwargs):
        """
        Initializes the OmniVerify client with SDK versioning for traceability.
        """
        sdk_version_origin = telesignenterprise.__version__
        sdk_version_dependency = telesign.__version__

        super(OmniVerify, self).__init__(
            customer_id,
            api_key,
            rest_endpoint=rest_endpoint,
            source=SOURCE_SDK,
            sdk_version_origin=sdk_version_origin,
            sdk_version_dependency=sdk_version_dependency,
            **kwargs
        )

    def createVerificationProcess(self, phone_number, params={}):
        """
        Create a verification process for the specified phone number.

        See https://developer.telesign.com/enterprise/reference/createverificationprocess for detailed API documentation.
        """
        params["recipient"] = {"phone_number": phone_number}

        if "verification_policy" not in params:
            params["verification_policy"] = [{"method": "sms"}]

        return self.post(PATH_VERIFICATION_CREATE, json_fields=params)    

    def getVerificationProcess(self, reference_id, params={}):
        """
        Retrieve details about the specified verification process.

        See https://developer.telesign.com/enterprise/reference/getverificationprocess for detailed API documentation.
        
        :param reference_id: The unique identifier of the verification process.
        :param params: Optional query parameters as a dictionary.
        :return: Response object from the GET request.
        """
        endpoint = PATH_VERIFICATION_RETRIEVE.format(reference_id=reference_id)
        headers = {"Content-Type": "application/json", "Accept": "application/json"}

        return self.get(endpoint, json_fields=params, headers=headers)    
    
    def updateVerificationProcess(self, reference_id, params, use_basic_auth=False):
        """
        Update a verification process.

        See https://developer.telesign.com/enterprise/reference/updateverificationprocess for detailed API documentation.

        :param reference_id: The unique identifier of the verification process.
        :param params: Dictionary of parameters for the update (must include 'action' and 'security_factor').
        :param use_basic_auth: Boolean indicating whether to use manual Basic Auth.
        :return: Response object.
        """
        endpoint_path = PATH_VERIFICATION_UPDATE.format(reference_id=reference_id)

        if use_basic_auth:
            endpoint = self.api_host.rstrip('/') + endpoint_path

            json_body = json.dumps(params)

            auth_str = f"{self.customer_id}:{self.api_key}"
            auth_bytes = auth_str.encode('utf-8')
            auth_b64 = base64.b64encode(auth_bytes).decode('utf-8')
            headers = {
                "Authorization": f"Basic {auth_b64}",
                "Content-Type": "application/json",
                "Accept": "application/json"
            }
            response = requests.patch(endpoint, data=json_body, headers=headers)
            return type('Response', (), {
                'status_code': response.status_code,
                'headers': response.headers,
                'body': response.text,
                'ok': response.ok,
                'json': response.json
            })()
        else:
            headers = {"Content-Type": "application/json", "Accept": "application/json"}
            return self.patch(endpoint_path, json_fields=params, headers=headers)


from pkg_resources import get_distribution

__import__('pkg_resources').declare_namespace(__name__)

__version__ = get_distribution("telesignenterprise").version
__author__ = "TeleSign"
__copyright__ = "Copyright 2017, TeleSign Corp."
__credits__ = ["TeleSign"]
__license__ = "MIT"
__maintainer__ = "TeleSign Corp."
__email__ = "support@telesign.com"
__status__ = "Production"



from __future__ import unicode_literals

from telesign.rest import RestClient
from telesignenterprise.constants import SOURCE_SDK
import telesignenterprise
import telesign
from telesignenterprise.omniverify import OmniVerify

VERIFY_SMS_RESOURCE = "/v1/verify/sms"
VERIFY_VOICE_RESOURCE = "/v1/verify/call"
VERIFY_SMART_RESOURCE = "/v1/verify/smart"
VERIFY_STATUS_RESOURCE = "/v1/verify/{reference_id}"
VERIFY_COMPLETION_RESOURCE = "/v1/verify/completion/{reference_id}"
BASE_URL_VERIFY_API = "https://verify.telesign.com"
DEFAULT_FS_BASE_URL = "https://rest-ww.telesign.com"
PATH_VERIFICATION = "/verification"


class VerifyClient(RestClient):
    """
    The Verify API delivers phone-based verification and two-factor authentication using a time-based, one-time passcode
    sent via SMS message or Voice call.
    """

    def __init__(
        self, customer_id, api_key, rest_endpoint=DEFAULT_FS_BASE_URL, **kwargs
    ):
        sdk_version_origin = telesignenterprise.__version__
        sdk_version_dependency = telesign.__version__
        super(VerifyClient, self).__init__(
            customer_id,
            api_key,
            rest_endpoint=rest_endpoint,
            source=SOURCE_SDK,
            sdk_version_origin=sdk_version_origin,
            sdk_version_dependency=sdk_version_dependency,
            **kwargs
        )
        # Instantiate OmniVerify for omnichannel methods
        self.omniverify = OmniVerify(customer_id, api_key)

    def createVerificationProcess(self, phone_number, params={}):
        
        return self.omniverify.createVerificationProcess(phone_number, params)  

    def sms(self, phone_number, **params):
        """
        The SMS Verify API delivers phone-based verification and two-factor authentication using a time-based,
        one-time passcode sent over SMS.

        See https://developer.telesign.com/docs/rest_api-verify-sms for detailed API documentation.
        """
        return self.post(VERIFY_SMS_RESOURCE, phone_number=phone_number, **params)

    def voice(self, phone_number, **params):
        """
        The Voice Verify API delivers patented phone-based verification and two-factor authentication using a one-time
        passcode sent over voice message.

        See https://developer.telesign.com/docs/rest_api-verify-call for detailed API documentation.
        """
        return self.post(VERIFY_VOICE_RESOURCE, phone_number=phone_number, **params)

    def smart(self, phone_number, ucid, **params):
        """
        The Smart Verify web service simplifies the process of verifying user identity by integrating several TeleSign
        web services into a single API call. This eliminates the need for you to make multiple calls to the TeleSign
        Verify resource.

        See https://developer.telesign.com/docs/rest_api-smart-verify for detailed API documentation.
        """
        return self.post(
            VERIFY_SMART_RESOURCE, phone_number=phone_number, ucid=ucid, **params
        )

    def status(self, reference_id, **params):
        """
        Retrieves the verification result for any verify resource.

        See https://developer.telesign.com/docs/rest_api-verify-transaction-callback for detailed API documentation.
        """
        return self.get(
            VERIFY_STATUS_RESOURCE.format(reference_id=reference_id), **params
        )

    def completion(self, reference_id, **params):
        """
        Notifies TeleSign that a verification was successfully delivered to the user in order to help improve
        the quality of message delivery routes.

        See https://developer.telesign.com/docs/completion-service-for-verify-products for detailed API documentation.
        """
        return self.put(
            VERIFY_COMPLETION_RESOURCE.format(reference_id=reference_id), **params
        )



from __future__ import unicode_literals

from telesign.appverify import AppVerifyClient as _AppVerifyClient
from telesignenterprise.constants import SOURCE_SDK
import telesignenterprise
import telesign


APP_VERIFY_INITIATE_RESOURCE = "/v1/verify/auto/voice/initiate"
APP_VERIFY_FINALIZE_RESOURCE = "/v1/verify/auto/voice/finalize"
APP_VERIFY_STATUS_RESOURCE = "/v1/verify/auto/voice/{}"


class AppVerifyClient(_AppVerifyClient):
    """
    The TeleSign App Verify web service enables customers to verify devices
    through a voice call by a verification code provided in the caller ID.
    """

    def __init__(
        self,
        customer_id,
        api_key,
        rest_endpoint="https://rest-ww.telesign.com",
        **kwargs
    ):
        sdk_version_origin = telesignenterprise.__version__
        sdk_version_dependency = telesign.__version__
        super(AppVerifyClient, self).__init__(
            customer_id,
            api_key,
            rest_endpoint=rest_endpoint,
            source=SOURCE_SDK,
            sdk_version_origin=sdk_version_origin,
            sdk_version_dependency=sdk_version_dependency,
            **kwargs
        )

    def initiate(self, phone_number, **params):
        """
        Make an Initiate request to TeleSign that includes the phone number used for the handset.
        TeleSign places a call to the handset. This call will be terminated by the finalize request,
        if not by the handset terminating the call in your application

        See https://enterprise.telesign.com/docs/app-verify-api for detailed API documentation.
        """
        return self.post(
            APP_VERIFY_INITIATE_RESOURCE, phone_number=phone_number, **params
        )

    def finalize(self, reference_id, **params):
        """
        TeleSign compares the verify code generated for the call with the verify code
        in the Finalize request to determine whether they match.
        The result indicates whether the transaction is successful or not ('success' or 'failed').

        See https://enterprise.telesign.com/docs/app-verify-api for detailed API documentation.
        """
        return self.post(
            APP_VERIFY_FINALIZE_RESOURCE, reference_id=reference_id, **params
        )

    def status(self, reference_id, **params):
        """
        TeleSign looks up the status for the verification call in the
        Initiate request to determine status of the AV call, including
        intermediate call status such as call failed, or retry.

        See https://enterprise.telesign.com/docs/app-verify-api for detailed API documentation.
        """
        return self.get(APP_VERIFY_STATUS_RESOURCE.format(reference_id), **params)
